from typing import TypedDict, List, Dict

from datacaster.classes import CastDataClass


class GraphClient(TypedDict):
    """
    - Contains Graph API connection details

    !!! info "GraphClient Keys"

        __app_id__: ID of the app registration in Microsoft Azure

        __client_secret__:  Application client secret

        __tenant_id__: Microsoft Azure tenant ID
    """

    app_id: str
    client_secret: str
    tenant_id: str


class UserData(CastDataClass):
    __class_config__ = {
        "rename_fields": {
            "displayName": "display_name",
            "givenName": "first_name",
            "jobTitle": "job_title",
            "officeLocation": "office_location",
            "userPrincipalName": "user_principal_name",
            "businessPhones": "business_phone_numbers",
            "preferredLanguage": "language",
        }
    }

    business_phone_numbers: List[str]
    display_name: str
    first_name: str
    surname: str
    id: str
    job_title: str
    language: str
    mail: str
    office_location: str
    user_principal_name: str


class User(TypedDict):
    """
    - Contains information about an Office 365 User

    !!! info "User Keys"

        __business_phone_numbers__: List of the user's phone numbers

        __display_name__:  User's display name (not unique)

        __first_name__: User's first name

        __surname__: User's surname

        __id__: User ID

        __job_title__: User's job title

        __language__: User's native language

        __mail__: User's email address

        __office_location__: User's office location

        __user_principal_name__: User's principal name
    """

    __annotations__ = UserData.__annotations__


class GroupData(CastDataClass):
    __class_config__ = {
        "rename_fields": {
            "displayName": "display_name",
            "groupTypes": "group_types",
            "mailEnabled": "mail_enabled",
            "mailNickname": "mail_nickname",
            "securityEnabled": "security_enabled",
        }
    }
    display_name: str
    description: str
    group_types: List[str] = None
    id: str
    mail_enabled: bool
    mail_nickname: str
    security_enabled: bool


class Group(TypedDict):
    """
    - Contains information about an Office 365 Group

    !!! info "Group Keys"

        __display_name__:  Group display name (not unique)

        __description__: Group description

        __group_types__: List of group types (only populated if the group is unified)

        __id__: Group ID

        __mail_enabled__: Indicates whether the group is mail-enabled

        __mail_nickname__: Group mail alias

        __security_enabled__: Indicates whether the group is a security group
    """

    __annotations__ = GroupData.__annotations__


class SubscriptionData(CastDataClass):
    __class_config__ = {
        "rename_fields": {
            "capabilityStatus": "capability_status",
            "consumedUnits": "consumed_units",
            "skuId": "sku_id",
            "skuPartNumber": "part_number",
            "appliesTo": "applies_to",
            "prepaidUnits": "prepaid_units",
        }
    }
    consumed_units: int
    id: str
    sku_id: str
    part_number: str
    prepaid_units: Dict[str, int]


class Subscription(TypedDict):
    """
    - Contains information about an Office 365 Subscription

    !!! info "Subscription Keys"

        __consumed_units__: Number of subscription licences currently assigned

        __id__:  Subscription ID

        __sku_id__: Subscription stock keeping unit ID

        __part_number__: Name of the subscription or product

        __prepaid_units__: Total number of purchased licences
    """

    __annotations__ = SubscriptionData.__annotations__
