"""
Subscriptions

This module contains actions for managing subscriptions in Office 365.
"""
import logging

from typing import List, Union, Optional

from ppa import ui
from playbook_tools import tables

from . import _client, users
from .records import GraphClient, Subscription, SubscriptionData, User


logger = logging.getLogger(__name__)


TABLE_HEADER = ["Part Number", "SKU ID", "Consumed Units", "Paid Units"]


def _row_generator(subscription):
    return (
        [
            subscription["part_number"].replace("_", " "),
            subscription["sku_id"],
            # The next two don't exist when getting subscriptions for a specific user.
            subscription.get("consumed_units") or "N/A",
            subscription.get("prepaid_units", {}).get("enabled") if subscription.get("prepaid_units") else "N/A",
        ]
        if subscription
        else []
    )


def _process_subscription(data) -> Subscription:
    return SubscriptionData(**data).__dict__


@_client.graph_api
def get_all(client: GraphClient) -> List[Subscription]:
    """Get all company subscriptions.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __client:__ a [GraphClient](../inputs_outputs/#graphclient)

    ??? success "Output"

         A list of [Subscriptions](../inputs_outputs/#subscription).

    ??? example
        ```YAML linenums="1"
        - microsoft_graph.subscriptions.get_all:
          load:
            client: graph_secrets
          save: subscriptions
        ```
    """
    logger.info("getting all subscriptions")
    with _client._exception_handler("get subscriptions"):
        return list(map(_process_subscription, _client.get(client, "/subscribedSkus")))


@_client.graph_api
def get_for_user(client: GraphClient, user_principal_name: str) -> List[Subscription]:
    """Get all subscriptions assigned to a user.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __client:__ a [GraphClient](../inputs_outputs/#graphclient)

        - __user_principal_name:__ the [User's](../inputs_outputs/#user) principal name

    ??? success "Output"

         A list of [Subscriptions](../inputs_outputs/#subscription).

    ??? example
        ```YAML linenums="1"
        - microsoft_graph.subscriptions.get_for_user:
          load:
            client: graph_secrets
          save: subscriptions
        ```
    """
    logger.info(f"getting subscriptions for user with principal name {user_principal_name}")
    with _client._exception_handler(f"get subscriptions for user '{user_principal_name}'"):
        return list(
            map(
                _process_subscription,
                _client.get(client, f"/users/{users.get_by_principal_name(client, user_principal_name)['id']}/licenseDetails"),
            )
        )


@_client.graph_api
def display_for_users(client: GraphClient, users: List[User]) -> None:
    """Generate & display a table of [Subscriptions](../inputs_outputs/#subscription) & the [Users](../inputs_outputs/#user) assigned to each.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __client:__ a [GraphClient](../inputs_outputs/#graphclient)

        - __users:__ a list of [Users](../inputs_outputs/#user) to get subscriptions for

    ??? success "Output"

         Nothing is outputted by this action.

    ??? example
        ```YAML linenums="1"
        - microsoft_graph.users.get_all:
          load:
            client: graph_secrets
          save: users

        - microsoft_graph.subscriptions.display_for_users:
          load:
            users: users
            client: graph_secrets
        ```
    """
    ui.output_progress(f"Generating licence report for {len(users)} users...", percent=None, element_id="generate_licence_report")
    logger.info(f"generating subscription report for {len(users)} users")
    subscription_tabs = {}
    for user in users:
        for subscription in get_for_user(client, user["id"]):
            part_number = subscription["part_number"].replace("_", " ")
            if licensed_users := subscription_tabs.get(part_number):
                licensed_users.append(user)
                subscription_tabs[part_number] = licensed_users
            else:
                subscription_tabs[part_number] = [user]

    if subscription_tabs == {}:
        ui.output_progress("No assigned licences were found", percent=100, element_id="generate_licence_report")
        ui.output_info("No licences are assigned to the selected users.")
    else:
        ui.output_progress(f"Generated licence report for {len(users)} users", percent=100, element_id="generate_licence_report")
        tables.output_tabbed_table(
            "Licences & Users",
            header=["Display Name", "User Principal Name", "Email Address"],
            extract=["display_name", "user_principal_name", "mail"],
            tabs=subscription_tabs,
        )


@_client.graph_api
def assign_to_user(client: GraphClient, user_principal_name: str, sku_id: str) -> None:
    """Assign a subscription licence to a user.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __client:__ a [GraphClient](../inputs_outputs/#graphclient)

        - __user_principal_name:__ the [User's](../inputs_outputs/#user) principal name

        - __sku_id:__ the subscription SKU ID

    ??? success "Output"

         Nothing is outputted by this action.

    ??? example
        ```YAML linenums="1"
        - microsoft_graph.subscriptions.assign_to_user:
            user_principal_name: john.smith@domain.com
          load:
            sku_id: subscription.sku_id
            client: graph_secrets
        ```
    """
    logger.info(f"assigning subscription {sku_id} to user with principal name {user_principal_name}")
    with _client._exception_handler(f"assign subscription licence to user '{user_principal_name}'"):
        _client.post(
            client,
            f"/users/{users.get_by_principal_name(client, user_principal_name)['id']}/assignLicense",
            json={"addLicenses": [{"skuId": sku_id, "disabledPlans": []}], "removeLicenses": []},
        )


@_client.graph_api
def remove_from_user(client: GraphClient, user_principal_name: str, sku_id: str) -> None:
    """Remove a subscription licence from a user.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __client:__ a [GraphClient](../inputs_outputs/#graphclient)

        - __user_principal_name:__ the [User's](../inputs_outputs/#user) principal name

        - __sku_id:__ the subscription SKU ID

    ??? success "Output"

         Nothing is outputted by this action.

    ??? example
        ```YAML linenums="1"
        - microsoft_graph.subscriptions.remove_from_user:
            user_principal_name: john.smith@domain.com
          load:
            sku_id: subscription.sku_id
            client: graph_secrets
        ```
    """
    logger.info(f"removing subscription {sku_id} from user with principal name {user_principal_name}")
    with _client._exception_handler(f"remove subscription licence from user '{user_principal_name}'"):
        _client.post(
            client,
            f"/users/{users.get_by_principal_name(client, user_principal_name)['id']}/assignLicense",
            json={"addLicenses": [], "removeLicenses": [sku_id]},
        )


def display(text: str, subscriptions: Union[List[Subscription], Subscription]) -> None:
    """Display subscriptions in a table.

The table will have the following columns:

- Part Number
- SKU ID
- Consumed Units
- Paid Units

!!! question "Minimum Plugin Version: 1.0.0"

??? info "Input"

    - __text:__ the title of the table

    - __subscriptions:__ any number of [Subscriptions](../inputs_outputs/#subscription)

??? success "Output"

    Nothing is outputted by this action.

??? example
    ```YAML linenums="1"
    - microsoft_graph.subscriptions.get_all:
      load:
        client: graph_secrets
      save: subscriptions

    - microsoft_graph.subscriptions.display:
        text: All Subscriptions
      load:
        subscriptions: subscriptions
    ```
    """
    tables.output_table(text, header=TABLE_HEADER, extract=_row_generator, records=subscriptions)


def select(
    text: str,
    subscriptions: Union[List[Subscription], Subscription],
    minimum: Optional[int] = None,
    maximum: Optional[int] = None,
) -> None:
    """Display subscriptions in a table & prompt the task operator to make a selection.

The table will have the following columns:

- Part Number
- SKU ID
- Consumed Units
- Paid Units

!!! question "Minimum Plugin Version: 1.0.0"

??? info "Input"

    - __text:__ the title of the table

    - __subscriptions:__ any number of [Subscriptions](../inputs_outputs/#subscription)

    - __minimum:__ the minimum number of selections

    - __maximum:__ the maximum number of selections

??? success "Output"

    A [Selection](../../../tables/#selection) containing:

    - **total**: the number of selected [Subscriptions](../inputs_outputs/#subscription)

    - **all**: a list of selected [Subscriptions](../inputs_outputs/#subscription)

    - **first**: the first selected [Subscriptions](../inputs_outputs/#subscription)

??? example
    ```YAML linenums="1"
    - microsoft_graph.subscriptions.get_all:
      load:
        client: graph_secrets
      save: subscriptions

    - microsoft_graph.subscriptions.select:
        text: Select Subscriptions
      load:
        subscriptions: subscriptions
      save: selection
    ```
    """
    return tables.input_table(
        text, header=TABLE_HEADER, extract=_row_generator, records=subscriptions, minimum=minimum, maximum=maximum
    )


def select_one(text: str, subscriptions: Union[List[Subscription], Subscription]) -> None:
    """Display subscriptions in a table & prompt the task operator to select one.

The table will have the following columns:

- Part Number
- SKU ID
- Consumed Units
- Paid Units

!!! question "Minimum Plugin Version: 1.0.0"

??? info "Input"

    - __text:__ the title of the table

    - __subscriptions:__ any number of [Subscriptions](../inputs_outputs/#subscription)

??? success "Output"

    A single [Subscription](../inputs_outputs/#subscription).

??? example

    ```YAML linenums="1"
    - microsoft_graph.subscriptions.get_all:
      load:
        client: graph_secrets
      save: subscriptions

    - microsoft_graph.subscriptions.select_one:
        text: Select a Subscription
      load:
        subscriptions: subscriptions
      save: subscription
    ```
    """
    return select(text, subscriptions=subscriptions, minimum=1, maximum=1)["first"]
