"""
Users

This module contains actions for managing users in Office 365.
"""
import logging

from typing import List, Union, Optional


from playbook_tools import tables


from . import _client
from .records import GraphClient, User, UserData


logger = logging.getLogger(__name__)


USER_TABLE_HEADER = ["Display Name", "User Principal Name", "Email Address", "Office Location"]
USER_TABLE_FIELDS = ["display_name", "user_principal_name", "mail", "office_location"]


def _process_user(data) -> User:
    return UserData(**data).__dict__


@_client.graph_api
def get_all(client: GraphClient) -> List[User]:
    """Get all users.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __client:__ a [GraphClient](../inputs_outputs/#graphclient)

    ??? success "Output"

         A list of [Users](../inputs_outputs/#user).

    ??? example
        ```YAML linenums="1"
        - microsoft_graph.users.get_all:
          load:
            client: graph_secrets
          save: users
        ```
    """
    logger.info("getting all users")
    with _client._exception_handler("get users"):
        return list(map(_process_user, _client.get(client, "/users")))


@_client.graph_api
def get_by_principal_name(client: GraphClient, user_principal_name: str) -> User:
    """Get the user with the supplied user principal name.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __client:__ a [GraphClient](../inputs_outputs/#graphclient)

        - __user_principal_name__: the [User's](../inputs_outputs/#user) principal name

    ??? success "Output"

         A single [User](../inputs_outputs/#user).

    ??? example
        ```YAML linenums="1"
        - microsoft_graph.users.get_by_principal_name:
            user_principal_name: john.smith@domain.com
          load:
            client: graph_secrets
          save: user
        ```
    """
    logger.info(f"getting user with principal name '{user_principal_name}'")
    with _client._exception_handler(f"get user '{user_principal_name}'"):
        return _process_user(_client.get(client, f"/users/{user_principal_name}"))


@_client.graph_api
def get_by_id(client: GraphClient, user_id: str) -> User:
    """Get the user with the supplied user ID.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __client:__ a [GraphClient](../inputs_outputs/#graphclient)

        - __user_id__: the [User](../inputs_outputs/#user) ID

    ??? success "Output"

         A single [User](../inputs_outputs/#user).

    ??? example
        ```YAML linenums="1"
        - microsoft_graph.users.get_by_id:
          load:
            user_id: group_member.id
            client: graph_secrets
          save: user
        ```
    """
    logger.info(f"getting user with ID '{user_id}'")
    with _client._exception_handler(f"get user with ID '{user_id}'"):
        return _process_user(_client.get(client, f"/users/{user_id}"))


@_client.graph_api
def create(
    client: GraphClient,
    display_name: str,
    mail_nickname: str,
    user_principal_name: str,
    password: str,
    enabled: Optional[bool] = True,
    force_password_change: Optional[bool] = False,
    force_password_change_mfa: Optional[bool] = False,
) -> None:
    """Create a user.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __client:__ a [GraphClient](../inputs_outputs/#graphclient)

        - __display_name:__ a display name for the user

        - __mail_nickname:__ a mail alias for the user

        - __user_principal_name__: the [User's](../inputs_outputs/#user) principal name

        - __password:__ a password to set for the user

        - __enabled:__ set to `false` to create the user in a disabled state (defaults to `true`)

        - __force_password_change:__ set to `true` to force password reset on first logon (defaults to `false`)

        - __force_password_change:__ set to `true` to force password reset _with MFA_ on first logon (defaults to `false`)

    ??? success "Output"

         Nothing is outputted by this action.

    ??? example
        ```YAML linenums="1"
        - microsoft_graph.users.create:
            display_name: John Smith
            user_principal_name: john.smith@domain.com
            mail: john.smith@domain.com
            mail_nickname: john.smith
            enabled: true
            password: "{{ new_password }}"
          load:
            client: graph_secrets
        ```
    """
    logger.info(f"creating user '{user_principal_name}'")
    if force_password_change is True and force_password_change_mfa is True:
        # If the MFA flag is supplied as True the other can safely be False (just to be safe & match the Graph API examples).
        force_password_change = False
    with _client._exception_handler(f"create user '{user_principal_name}'"):
        return _client.post(
            client,
            "/users",
            json={
                "displayName": display_name,
                "mailNickname": mail_nickname,
                "userPrincipalName": user_principal_name,
                "accountEnabled": enabled,
                "passwordProfile": {
                    "forceChangePasswordNextSignIn": force_password_change,
                    "forceChangePasswordNextSignInWithMfa": force_password_change_mfa,
                    "password": password,
                },
            },
        )


@_client.graph_api
def delete(client: GraphClient, user_principal_name: str) -> None:
    """Delete a user.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __client:__ a [GraphClient](../inputs_outputs/#graphclient)

        - __user_principal_name__: the [User's](../inputs_outputs/#user) principal name

    ??? success "Output"

         Nothing is outputted by this action.

    ??? example
        ```YAML linenums="1"
        - microsoft_graph.users.delete:
            user_principal_name: john.smith@domain.com
          load:
            client: graph_secrets
        ```
    """
    logger.info(f"deleting user '{user_principal_name}'")
    with _client._exception_handler(f"delete user '{user_principal_name}'"):
        return _client.delete(client, f"/users/{user_principal_name}", json=None)


def display(text: str, users: Union[List[User], User]) -> None:
    """Display users in a table.

The table will have the following columns:

- Display Name
- User Principal Name
- Email Address
- Office Location

!!! question "Minimum Plugin Version: 1.0.0"

??? info "Input"

    - __text:__ the title of the table

    - __users:__ any number of [Users](../inputs_outputs/#user)

??? success "Output"

    Nothing is outputted by this action.

??? example
    ```YAML linenums="1"
    - microsoft_graph.users.get_all:
      load:
        client: graph_secrets
      save: users

    - microsoft_graph.users.display:
        text: All Users
      load:
        users: users
    ```
    """
    tables.output_table(text, header=USER_TABLE_HEADER, extract=USER_TABLE_FIELDS, records=users)


def select(text: str, users: Union[List[User], User], minimum: Optional[int] = None, maximum: Optional[int] = None) -> None:
    """Display users in a table & prompt the task operator to make a selection.

The table will have the following columns:

- Display Name
- User Principal Name
- Email Address
- Office Location

!!! question "Minimum Plugin Version: 1.0.0"

??? info "Input"

    - __text:__ the title of the table

    - __users:__ any number of [Users](../inputs_outputs/#user)

    - __minimum:__ the minimum number of selections

    - __maximum:__ the maximum number of selections

??? success "Output"

    A [Selection](../../../tables/#selection) containing:

    - **total**: the number of selected [Users](../inputs_outputs/#user)

    - **all**: a list of selected [Users](../inputs_outputs/#user)

    - **first**: the first selected [User](../inputs_outputs/#user)

??? example

    ```YAML linenums="1"
    - microsoft_graph.users.get_all:
      load:
        client: graph_secrets
      save: users

    - microsoft_graph.users.select:
        text: Select Users
      load:
        users: users
      save: selection
    ```
    """
    return tables.input_table(
        text, header=USER_TABLE_HEADER, extract=USER_TABLE_FIELDS, records=users, minimum=minimum, maximum=maximum
    )


def select_one(text: str, users: Union[List[User], User]) -> None:
    """Display users in a table & prompt the task operator to select one.

The table will have the following columns:

- Display Name
- User Principal Name
- Email Address
- Office Location

!!! question "Minimum Plugin Version: 1.0.0"

??? info "Input"

    - __text:__ the title of the table

    - __users:__ any number of [Users](../inputs_outputs/#user)

??? success "Output"

    A single [User](../inputs_outputs/#user).

??? example

    ```YAML linenums="1"
    - microsoft_graph.users.get_all:
      load:
        client: graph_secrets
      save: users

    - microsoft_graph.users.select_one:
        text: Select a User
      load:
        users: users
      save: user
    ```
    """
    return select(text, users=users, minimum=1, maximum=1)["first"]
